#include <QtCore>
#include <QtNetwork>
#include <QNetworkReply>
#include "httpwindow.h"

HttpWindow::HttpWindow(QObject *parent)
    : QObject(parent)
{
    connect(&qnam, SIGNAL(finished (QNetworkReply*)),
            this, SLOT(replyFinished(QNetworkReply*)));
    connect(&qnam, SIGNAL(networkAccessibleChanged ( QNetworkAccessManager::NetworkAccessibility )),
            this, SLOT(networkAccessibleChanged ( QNetworkAccessManager::NetworkAccessibility )));
    connect(&qnam, SIGNAL(authenticationRequired(QNetworkReply*,QAuthenticator*)),
            this, SLOT(slotAuthenticationRequired(QNetworkReply*,QAuthenticator*)));
#ifndef QT_NO_OPENSSL
    connect(&qnam, SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)),
            this, SLOT(sslErrors(QNetworkReply*,QList<QSslError>)));
#endif
}

void HttpWindow::startRequest(QUrl url)
{
    reply = qnam.get(QNetworkRequest(url));
    connect(reply, SIGNAL(finished()),
            this, SLOT(httpFinished()));
    connect(reply, SIGNAL(readyRead()),
            this, SLOT(httpReadyRead()));
    connect(reply, SIGNAL(downloadProgress(qint64,qint64)),
            this, SLOT(updateDataReadProgress(qint64,qint64)));

    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)),
            this, SLOT(replyError(QNetworkReply::NetworkError)));
}
void HttpWindow::replyError ( QNetworkReply::NetworkError code ) {
    qCritical() <<"reply error:"<<code<<reply->errorString();
}

void HttpWindow::httpFinished()
{
    QVariant redirectionTarget = reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
    if (reply->error()) {
        qCritical()<<reply->errorString();
    } else if (!redirectionTarget.isNull()) {        
        QUrl newUrl = reply->url().resolved(redirectionTarget.toUrl());
        qCritical()<<"redirect: "<<newUrl.toString();
        reply->deleteLater();
        startRequest(newUrl);
        return;
    } else {
        qCritical() << "finished.";
    }

    reply->deleteLater();
    reply = 0;
}

void HttpWindow::httpReadyRead()
{
    // this slot gets called every time the QNetworkReply has new data.
    // We read all of its new data and write it into the file.
    // That way we use less RAM than when reading it at the finished()
    // signal of the QNetworkReply
    qCritical()<<"readyRead:"<<reply->readAll();
}

void HttpWindow::updateDataReadProgress(qint64 bytesRead, qint64 totalBytes)
{
    qCritical()<<"progress:"<<bytesRead<<"/"<<totalBytes;
}

void HttpWindow::slotAuthenticationRequired(QNetworkReply*,QAuthenticator *authenticator)
{
    qCritical()<<"auth required";
    authenticator->setUser("foo");
    authenticator->setPassword("bar");
}

#ifndef QT_NO_OPENSSL
void HttpWindow::sslErrors(QNetworkReply*,const QList<QSslError> &errors)
{
    foreach (const QSslError &error, errors) {
        qCritical()<< error.errorString();
    }
}
#endif
void HttpWindow::replyFinished(QNetworkReply* r) {
    qCritical()<<"QNAM: reply finished"<<r->errorString();
}

void HttpWindow::networkAccessibleChanged ( QNetworkAccessManager::NetworkAccessibility a) {
    qCritical()<<"accessability changed:"<<a;
}
